local addonName, addon = ...;

local function createCheckBox (frameName, text, parent, anchor, anchorParent, posX, poxY, func)
    local checkButton = CreateFrame('CheckButton', frameName, parent, 'OptionsCheckButtonTemplate')
    checkButton:SetChecked(addon.settings[frameName]);
    checkButton:SetPoint(anchor, parent, anchorParent, posX, poxY);
    _G[frameName .. "Text"]:SetText(text);
    _G[frameName .. "Text"]:SetJustifyH('LEFT');

    checkButton.SetValue = function()
        addon.settings[frameName] = checkButton:GetChecked();
        settings = addon.settings;
        if (func ~= nil) then
            func();
        end
    end

    return checkButton
end

local function createDropDown(frameName, parent, anchor, anchorParent, posX, posY, options, func)
    local dropDown = CreateFrame('Frame', frameName, parent, 'UIDropDownMenuTemplate');
    dropDown:SetPoint(anchor, rmSettingsFrame, anchorParent, posX, posY);
    UIDropDownMenu_SetWidth(dropDown, 138);

    UIDropDownMenu_Initialize(dropDown, function(self, level, menuList)
        local currentValue = addon.settings[frameName];
        for i = 1, #options do
            local option = options[i];
            local info = UIDropDownMenu_CreateInfo();

            info.text = option.TEXT;
            info.value = option.VALUE;
            info.func = function(self)
                currentValue = self.value;
                addon.settings[frameName] = currentValue;
                settings = addon.settings;
                UIDropDownMenu_SetSelectedValue(dropDown, self.value)
                if (func ~= nil) then
                    func();
                end
            end
            info.checked = (currentValue == option.VALUE);
            UIDropDownMenu_AddButton(info, level);
            if (options[i].VALUE == currentValue) then
                UIDropDownMenu_SetSelectedValue(self, options[i].VALUE)
            end
        end
    end);

    return dropDown;
end

local function createEditBox(frameName, text, parent, anchor, anchorParent, posX, poxY, width, height, func)
    local scrollFrame = CreateFrame("ScrollFrame", frameName, parent, "UIPanelScrollFrameTemplate")
    scrollFrame:SetPoint(anchor, parent, anchorParent, posX, poxY);
    scrollFrame:SetSize(width, height)

    local editbox = CreateFrame("EditBox", nil, scrollFrame)
    editbox:SetMultiLine(true)
    editbox:SetFontObject(ChatFontNormal)
    editbox:SetWidth(width)

    scrollFrame:SetScrollChild(editbox)

    editbox:SetText(table.concat(text, "\n"))

    -- The below will highlight all the text in the editbox. This is useful for
    -- dialogs where you want the user to copy something, such as a URL.
    -- editbox:HighlightText()
    editbox:SetScript("OnCursorChanged", func)
end

local function createTable(frameName, text, parent, anchor, anchorParent, posX, poxY, width, height, func)
	local scrollFrame = CreateFrame("ScrollFrame", frameName, parent, "UIPanelScrollFrameTemplate")
    scrollFrame:SetPoint(anchor, parent, anchorParent, posX, poxY);
    scrollFrame:SetSize(width, height)
	
	local editbox = CreateFrame("EditBox", nil, scrollFrame)
    editbox:SetMultiLine(true)
    editbox:SetFontObject(ChatFontNormal)
    editbox:SetWidth(width)

    scrollFrame:SetScrollChild(editbox)

    editbox:SetText(table.concat(text, "\n"))

    -- The below will highlight all the text in the editbox. This is useful for
    -- dialogs where you want the user to copy something, such as a URL.
    -- editbox:HighlightText()
    editbox:SetScript("OnCursorChanged", func)

end

addon.createCheckBox = createCheckBox
addon.createDropDown = createDropDown
addon.createEditBox = createEditBox
addon.createTable = createTable