local addonName, addon = ...;

local MountJournalFrame = CreateFrame('Frame', 'MountJournalFrame');

local events = {};

local init = false;

local L = setmetatable({}, {
    __index = function(table, key)
        return key;
    end,
});

function MountJournalFrame:registerEvents(events)
    for key, value in pairs(events) do
        MountJournalFrame:RegisterEvent(key);
    end

    local function eventHandler (self, event, ...)
        events[event](self, ...);
    end

    MountJournalFrame:SetScript('OnEvent', eventHandler);
end

local function toggleShowFavorites()
    _G['customFavorites']:SetShown(addon.settings['showCustomFavList']);
    _G['MountJournal'].MountDisplay:SetShown(not addon.settings['showCustomFavList'])
end

local function Eventhandler(self)
    local customMountList = {}
    for i in string.gmatch(self:GetText(), "[0-9]+") do
        table.insert(customMountList, i)
    end
    addon.customMountList = customMountList
end

local function CreateMountIdDisplay()
    local frame=CreateFrame("Frame","MountId",MountJournal);
    frame:SetPoint("TOP");
    frame:SetSize(100,100);
    local text=frame:CreateFontString("MountIdText","OVERLAY","GameFontNormal");--    Our text area
    text:SetPoint("CENTER");
    text:SetText(MountJournal.selectedMountID);
end

local function createMountJournalUI()
    addon.createEditBox('customFavorites', addon.customMountList, MountJournal, "TOPRIGHT", "TOPRIGHT", -29 , -63, 385, CollectionsJournal:GetHeight()-88, Eventhandler);
    addon.createCheckBox('showCustomFavList', L['show custom favorites'], MountJournal, 'BOTTOMLEFT', 'BOTTOMLEFT', 150, 1, toggleShowFavorites);
    CreateMountIdDisplay();
end

local function initClickHandler ()
    hooksecurefunc('MountJournal_UpdateMountDisplay', function ()
        _G['MountIdText']:SetText(MountJournal.selectedMountID)
    end)
end

function events:ADDON_LOADED (loadedAddon)
    if (loadedAddon == addonName and IsAddOnLoaded('Blizzard_Collections')) or
       (loadedAddon == 'Blizzard_Collections' and IsAddOnLoaded(addonName)) then
        createMountJournalUI();
        toggleShowFavorites();
        initClickHandler();
        init = true;
    end
end

MountJournalFrame:registerEvents(events);
