local addonName, addon                = ...;
local events                          = {}
local RandomMountFrame                = CreateFrame('Frame', 'RandomMountFrame');

local WOTLK_FLYING_SKILL              = { 54197 };
local CATA_FLYING_SKILL               = { 34090, 34091, 90265 };
local MOP_FLIYING_SKILL               = { 191645 };
local WOD_FLYING_SKILL                = { 115913 };
local LEGION_FLYING_SKILL             = { 233368 };
local BFA_FLYING_SKILL                = { 278833 };
local SL_FLYING_SKILL				  = { 352177 };

local FLIGHT_SKILLS                   = {
  [12]   = CATA_FLYING_SKILL,
  [13]   = CATA_FLYING_SKILL,
  [207]  = CATA_FLYING_SKILL,
  [101]  = CATA_FLYING_SKILL,
  [113]  = WOTLK_FLYING_SKILL,
  [81]   = LEGION_FLYING_SKILL,
  [619]  = LEGION_FLYING_SKILL,
  [14]   = BFA_FLYING_SKILL,
  [18]   = BFA_FLYING_SKILL,
  [62]   = BFA_FLYING_SKILL,
  [875]  = BFA_FLYING_SKILL,
  [876]  = BFA_FLYING_SKILL,
  [1355] = BFA_FLYING_SKILL,
  [1527] = BFA_FLYING_SKILL,
  [1530] = BFA_FLYING_SKILL,
  [424]  = MOP_FLIYING_SKILL,
  [17]   = WOD_FLYING_SKILL,
  [572]  = WOD_FLYING_SKILL,
  [1550] = SL_FLYING_SKILL,
};

local TEMPLE_OF_AHN_QIRAJI            = { [319] = true,
                                          [320] = true,
                                          [321] = true,
                                          [435] = true,
};

local IGNORE_PARENT_MAP_BY_DIFFICULTY = { [435] = 2, };

local IGNORE_PARENT_MAP_BY_BUFF       = { [17]   = true,
                                          [18]   = true,
                                          [81]   = true,
                                          [1527] = true,
                                          [1530] = true,
};

local TIMEWALKER_SPELL_IDS            = { [123979] = true,
                                          [176111] = true,
                                          [176113] = true,
                                          [254169] = true,
                                          [255152] = true,
                                          [264606] = true,
                                          [276824] = true,
                                          [276827] = true,
                                          [276950] = true,
                                          [276954] = true,
                                          [277427] = true,
                                          [277430] = true,
                                          [290245] = true,
                                          [290246] = true,
                                          [316193] = true,
                                          [317785] = true,
                                          [317928] = true,
                                          [317931] = true,
};

local CONTROLLED_BY_FACTION           = { [281114] = true,
                                          [281115] = true,
                                          [281116] = true,
                                          [281117] = true,
};
local FLYABLE_AREA_BLACKLIST = {[328] = true,
                                [435] = true,
                                [1355] = true,};

local isUnderwater = false

local function openOptions()
  InterfaceOptionsFrame_Show()
  InterfaceOptionsFrame_OpenToCategory(_G['rmSettingsFrame'])
end

local function ignoreParentId(uiMapID)

  if (TEMPLE_OF_AHN_QIRAJI[uiMapID]) then
    return true;
  end

  local timewalkingActive   = false;
  local controlledbyFaction = false;
  for i = 1, 40 do
    repeat
      local _, _, _, _, _, _, _, _, _, spellId = UnitBuff("player", i);
      if (spellId == nil) then
        do break end
      end

      if (TIMEWALKER_SPELL_IDS[spellId]) then
        timewalkingActive = true;
      end
      if (CONTROLLED_BY_FACTION[spellId]) then
        controlledbyFaction = true;
      end
    until true;
  end

  if (timewalkingActive) then
    return false
  end

  if (((uiMapID == 62 or uiMapID == 14) and controlledbyFaction) or IGNORE_PARENT_MAP_BY_BUFF[uiMapID]) then
    return true;
  end
  if (addon.zoneToExpansion[uiMapID]) then
    return true;
  end
  return false;
end

local function GetParentMap(uiMapID)
  local mapinfo = C_Map.GetMapInfo(uiMapID);
  return mapinfo.parentMapID;
end

local function numZoneToExpansion(zoneId)
  local x = 0
  for _,_ in pairs(addon.zoneToExpansion[zoneId]) do
    x = x + 1
  end
  return x
end

local function GetCurrentExpansion(zoneId, forceExpansionID)
  if (addon.zoneToExpansion[zoneId] == nil) then return nil end

  if (forceExpansionID ~= nil and numZoneToExpansion(zoneId) > 1) then
    for key, val in pairs(addon.zoneToExpansion[zoneId]) do
      if (val == false and key == forceExpansionID) then
        return key;
      end
    end
  end
  if (addon.zoneToExpansion[zoneId] == 1) then
    return addon.zoneToExpansion[zoneId];
  else
    for key, val in pairs(addon.zoneToExpansion[zoneId]) do
      if (val == true) then
        return key;
      end
    end
  end
  return nil;
end

local function GetCurrentContinentID()
  local uiMapID = C_Map.GetBestMapForUnit("player");
  if ignoreParentId(uiMapID) then
    local mapinfo = C_Map.GetMapInfo(uiMapID)
    return mapinfo.mapID;
  end
  local parentMapId = GetParentMap(uiMapID)
  local mapinfo     = C_Map.GetMapInfo(parentMapId);
  if mapinfo ~= nil then
    while mapinfo.mapType > 2 do
      parentMapId = GetParentMap(mapinfo.mapID)
      mapinfo     = C_Map.GetMapInfo(parentMapId);
    end
  end
  if mapinfo == nil or mapinfo.mapID == nil or mapinfo.mapType < 2 then
    return uiMapID;
  end
  return mapinfo.mapID;
end

local function CanPlayerFly(continentId)
  local canFly = false;
  if (FLIGHT_SKILLS[continentId] == nil) then
    return false
  end
  for x = 1, #FLIGHT_SKILLS[continentId], 1 do
    if not canFly then
      canFly = FLIGHT_SKILLS[continentId][x]
    end
  end

  if (FLYABLE_AREA_BLACKLIST[continentId]) then
    return false;
  end
  if (IsSpellKnown(34090) or IsSpellKnown(34091) or IsSpellKnown(90265)) and canFly then
    return true;
  end

  return false;
end

local function CanPlayerRide()
  if (IsSpellKnown(33388) or IsSpellKnown(33391) or IsSpellKnown(34090) or IsSpellKnown(34091) or IsSpellKnown(90265)) then
    return true;
  end
  return false;
end

local function IsSwimmingPossible(playerStateMap, mountTypeID, continentId)
  if (not CanPlayerRide()) then return false end
  if ((((playerStateMap.isSubmerged and playerStateMap.isUnderwater) or continentId == 1355) and mountTypeID == 254) and not addon.settings['grmlWTHCheckButton']) then
    return true
  end
  return false
end

local function IsFlyingPossible(playerStateMap, mountTypeID, continentId)
  if (not CanPlayerRide()) then return false end
  local flyableArea    = (addon.settings['rmModeDropdown'] > 2 and playerStateMap.isFlyableArea)
  local flyingPossible = (flyableArea and (not playerStateMap.isSubmerged or not playerStateMap.isUnderwater) and CanPlayerFly(continentId) and (mountTypeID == 248 or mountTypeID == 242))
  if ((flyingPossible or (continentId == 1355 and flyingPossible)) and not addon.settings['grmlWTHCheckButton']) then
    return true
  end
  return false
end

local function IsRidingPossibleInQiraji(playerStateMap, mountTypeID, continentId)
  if (not CanPlayerRide()) then return false end
  local isFlyingAllowed = (addon.settings['rmModeDropdown'] > 2 and playerStateMap.isFlyableArea)

  local ridingPossible  = ((not playerStateMap.isSubmerged or not playerStateMap.isUnderwater) and (not CanPlayerFly(continentId) or not isFlyingAllowed)
      and mountTypeID == 241 and TEMPLE_OF_AHN_QIRAJI[continentId])
  if (ridingPossible and not addon.settings['grmlWTHCheckButton']) then
    return true
  end
  return false
end

local function IsRidingPossible(playerStateMap, mountTypeID, continentId)
  if (not CanPlayerRide()) then return false end
  local isFlyingAllowed      = (addon.settings['rmModeDropdown'] > 2)
  local considerFlyingMounts = (addon.settings['flyingMountsAsGroundCheckButton'] and mountTypeID == 248 and isFlyingAllowed)
  local ridingPossible       = ((not playerStateMap.isSubmerged or not playerStateMap.isUnderwater) 
      and (mountTypeID == 230 and not playerStateMap.isFlyableArea or (considerFlyingMounts or (not CanPlayerFly(continentId) and not playerStateMap.isFlyableArea))) and continentId ~= 1355 and continentId ~= 320)
  if ((ridingPossible or (continentId == 1355 and ridingPossible)) and not addon.settings['grmlWTHCheckButton']) then
    return true
  end
  return false
end

local function filterList(playerStateMap, mountsToSummonList, mounts, mountTypeID, continentId, mountID)
  if (IsSwimmingPossible(playerStateMap, mountTypeID, continentId)) then
    table.insert(mountsToSummonList, mountID);
  elseif (IsFlyingPossible(playerStateMap, mountTypeID, continentId)) then
    if mounts[mountID] then
      table.insert(mountsToSummonList, mountID);
    end
  elseif (IsRidingPossible(playerStateMap, mountTypeID, continentId)) then
    if mounts[mountID] then
      table.insert(mountsToSummonList, mountID);
    end
  elseif (IsRidingPossibleInQiraji(playerStateMap, mountTypeID, continentId)) then
    if mounts[mountID] then
      table.insert(mountsToSummonList, mountID);
    end
  elseif ((addon.settings['grmlWTHCheckButton']
    or not CanPlayerRide())
    and mountTypeID == 284) then
    if mounts[mountID] then
      table.insert(mountsToSummonList, mountID);
    end
  end
end

local function determineMountsToSummonListByPassAddonCategory(playerStateMap)
  local continentId       = GetCurrentContinentID();
  local mountsToSummonList = {};
  local mountList = addon.mountlist;
  for _, mounts in pairs(mountList) do
    for _, val in pairs(mounts) do
	  for mountId, _ in pairs(val) do
	    local _ , _, _, _, isUsable, _, _, _, _, _, _, mountID = C_MountJournal.GetMountInfoByID(mountId);
        if isUsable then
          local _, _, _, _, mountTypeID, _, _, _, _ = C_MountJournal.GetMountInfoExtraByID(mountID);
	  
          filterList(playerStateMap, mountsToSummonList, val, mountTypeID, continentId, mountID)
        end
	  end
	end
  end
  return mountsToSummonList;
end

local function determineMountsToSummonList(playerStateMap, mountIds, hasFavoriteMounts, mapId, zoneToSpecialMount)
  local continentId       = GetCurrentContinentID();
  local expansionId = GetCurrentExpansion(continentId);
  local mountsToSummonList = {};
  local mountList = addon.mountlist;
  for key, val in pairs(mountIds) do 
    repeat
      local name , _, _, _, isUsable, _, isFavorite, isFactionSpecific, _, shouldHideOnChar, isCollected, mountID = C_MountJournal.GetMountInfoByID(val);
      if isUsable and not addon.settings['w'] then
		
        local _, _, _, _, mountTypeID, _, _, _, _ = C_MountJournal.GetMountInfoExtraByID(mountID);
        if mountList[expansionId] == nil then
          do
            break
          end
        end
        if mountList[expansionId][mountTypeID] == nil then
          do
            break
          end
        end
		
		local mounts = nil;
		if (zoneToSpecialMount[mapId] ~= nil) then
			mounts = zoneToSpecialMount[mapId];
		else
			mounts = mountList[expansionId][mountTypeID];
		end
		
		filterList(playerStateMap, mountsToSummonList, mounts, mountTypeID, continentId, mountID)
      end
    until true
  end
  return mountsToSummonList;
end

local function IsUnderwater()
  return isUnderwater
end

local function RandomMount()
  if (addon.settings['rmModeDropdown'] == 1) then
    return
  end
  local mountsToSummonList = {};
  local hasFavoriteMounts = false;
  local continentId       = GetCurrentContinentID();
  local expansionId = GetCurrentExpansion(continentId);
  local mapId = C_Map.GetBestMapForUnit("player");
  local zoneToSpecialMount = addon.zoneToSpecialMount;
  local playerStateMap = {
    isSubmerged = false,
	isUnderwater, false,
	isFlyableArea = false,
  }
  playerStateMap.isSubmerged = IsSubmerged()
  playerStateMap.isUnderwater = IsUnderwater()
  playerStateMap.isFlyableArea = IsFlyableArea()
  
  local _, _, difficultyID, _, _, _, _, _, _, _ = GetInstanceInfo();
  if (IGNORE_PARENT_MAP_BY_DIFFICULTY[continentId] == difficultyID) then

    expansionId = GetCurrentExpansion(continentId, addon.expansions.MOP);
  end
  local mountIds = {}

  if addon.settings['useCustomFavListButton'] then
    mountIds = addon.customMountList;
  else
    mountIds = C_MountJournal.GetMountIDs();
  end

  if addon.settings['disableRandomMounter'] then
    mountsToSummonList = determineMountsToSummonListByPassAddonCategory(playerStateMap)
  else
    mountsToSummonList = determineMountsToSummonList(playerStateMap, mountIds, hasFavoriteMounts, mapId, zoneToSpecialMount)
  end
    
  if hasFavoriteMounts and continentId ~= 1355 then
    C_MountJournal.SummonByID(0)
  else
    local numMounts = table.getn(mountsToSummonList);
    if numMounts > 0 then
      local index = fastrandom(1, numMounts);
      C_MountJournal.SummonByID(mountsToSummonList[index])
    else
      C_MountJournal.SummonByID(0)
    end
  end
end

local function createRandomMountButton()
  local randomMountButton = CreateFrame('Button', "RandomMountButton", UIParent, 'SecureHandlerClickTemplate');
  randomMountButton:SetWidth(addon.settings['randomMountButtonSizeDropdown']);
  randomMountButton:SetHeight(addon.settings['randomMountButtonSizeDropdown']);
  randomMountButton:RegisterForClicks("AnyUp");
  randomMountButton:SetPoint(unpack(addon.settings['RandomMountButton'].anchor));
  randomMountButton:SetScript("OnClick", RandomMount);
  randomMountButton:SetNormalTexture(413588)
  randomMountButton:SetPushedTexture(413588)
  randomMountButton:SetHighlightTexture(413588)
  randomMountButton:SetShown(addon.settings['showRandomMountButton']);
  randomMountButton:SetMovable(not addon.settings['lockRandomMountButton'])
  randomMountButton:SetScript("OnDragStart", function()
    if (not addon.settings['lockRandomMountButton']) then
      randomMountButton:StartMoving();
    end
  end);
  randomMountButton:SetScript("OnDragStop", function()
    if (not addon.settings['lockRandomMountButton']) then
      randomMountButton:StopMovingOrSizing();
      addon.settings['RandomMountButton'].anchor = { randomMountButton:GetPoint() };
    end
  end);
  randomMountButton:RegisterForDrag("RightButton")
  randomMountButton:RegisterForClicks("LeftButtonUp");
end

local function createDebugButtonOptions()
  local debugButtonOptions = CreateFrame('Button', "DebugButtonOptions", UIParent, 'SecureHandlerClickTemplate');
  debugButtonOptions:SetWidth(64);
  debugButtonOptions:SetHeight(64);
  debugButtonOptions:RegisterForClicks("AnyUp");
  debugButtonOptions:SetPoint(unpack({ "CENTER", nil, "CENTER", 400, 400 }));
  debugButtonOptions:SetScript("OnClick", openOptions);
  debugButtonOptions:SetNormalTexture(413588)
  debugButtonOptions:SetPushedTexture(413588)
  debugButtonOptions:SetHighlightTexture(413588)
  debugButtonOptions:SetShown(addon.settings['enableDebug']);

  debugButtonOptions:RegisterForDrag("RightButton")
  debugButtonOptions:RegisterForClicks("LeftButtonUp");
end

local function DumpMountList()
  dumpMountList = {}
  for key, val in pairs(C_MountJournal.GetMountIDs()) do
      local _, _, _, _, isUsable, _, isFavorite, _, _, _, _, mountID = C_MountJournal.GetMountInfoByID(val);
      local _, _, _, _, mountTypeID, _, _, _, _ = C_MountJournal.GetMountInfoExtraByID(mountID);

      dumpMountList[mountID] = mountTypeID
  end
end

local function dumpMountByIds()

end

local function initSettings()
  if  customMountList == nil then
    customMountList = {};
  end
  if settings == nil then
    settings = {};
  end
  if (settings['rmModeDropdown'] == nil) then
    settings['rmModeDropdown'] = 3;
  end
  if (settings['grmlWTHCheckButton'] == nil) then
    settings['grmlWTHCheckButton'] = false;
  end
  if (settings['flyingMountsAsGroundCheckButton'] == nil) then
    settings['flyingMountsAsGroundCheckButton'] = false;
  end
  if (settings['randomMountButtonSizeDropdown'] == nil) then
    settings['randomMountButtonSizeDropdown'] = 64;
  end

  if (settings['lockRandomMountButton'] == nil) then
    settings['lockRandomMountButton'] = true;
  end
  if (settings['showRandomMountButton'] == nil) then
    settings['showRandomMountButton'] = true;
  end
  if (settings['RandomMountButton'] == nil) then
    settings['RandomMountButton'] = { anchor = { "CENTER", nil, "CENTER", 0, 0 }, };
  end

  addon.customMountList = customMountList;
  addon.settings = settings;
end

function events:ADDON_LOADED (loadedAddon)
  if loadedAddon == addonName then
    initSettings()
  end
end

function events:PLAYER_LOGIN ()
  createRandomMountButton()

  createDebugButtonOptions()
  -- DumpMountList()
end

function events:MIRROR_TIMER_START (arg1)
  if (arg1=="BREATH") then
      isUnderwater = true
  end
end

function events:MIRROR_TIMER_STOP (arg1)
  if (arg1=="BREATH") then
      isUnderwater = false
  end
end 

function events:PLAYER_LOGOUT ()
  customMountList = addon.customMountList
  settings = addon.settings
end

function RandomMountFrame:registerEvents(events)
  for key, value in pairs(events) do
    RandomMountFrame:RegisterEvent(key);
  end

  local function eventHandler (self, event, ...)
    events[event](self, ...);
  end

  RandomMountFrame:SetScript('OnEvent', eventHandler);
end

RandomMountFrame:registerEvents(events);
_G['SLASH_' .. addonName .. '1'] = "/randommounter";
_G['SLASH_' .. addonName .. '2'] = "/rm";
SlashCmdList[addonName]          = function(msg)
  if (msg == "") then
    RandomMount()
  elseif (msg == "options") then
    openOptions()
  end
end
